<?php
namespace CRTPLSTCreativeProductsList;

define( "CRTPLST_ASFSK_ASSETS_ADMIN_DIR_FILE", plugin_dir_url( __FILE__ ) . "assets/admin" );
class CRTPLSTCreativeListProductsClass {

	private static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function crtplst_admin_editor_scripts() {
		add_filter( 'script_loader_tag', [ $this, 'crtplst_admin_editor_scripts_as_a_module' ], 10, 2 );
	}

	public function crtplst_admin_editor_scripts_as_a_module( $tag, $handle ) {
		if ( 'crtplst_the_service_editor' === $handle ) {
			$tag = str_replace( '<script', '<script type="module"', $tag );
		}
		return $tag;
	}

	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/crtplst-products.php' );
	}

	public function crtplst_register_widgets() {
		// Its is now safe to include Widgets files
		$this->include_widgets_files();

		// Register Widgets
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\CRTPLSTProductsList() );
	}

	// Register Category
	public function crtplst_add_elementor_widget_categories( $elements_manager ) {

		$elements_manager->add_category(
			'bwdthebest_woocommerce_category',
			[
				'title' => esc_html__( 'BWD WooCommerce Group', 'creative-products-list' ),
				'icon' => 'eicon-person',
			]
		);
	}

	public function crtplst_all_assets_for_the_public(){
		wp_enqueue_style( 'crtplst-main-style', 'https://bwd-elementor-addons-pro.netlify.app/product-list.css', null, '2.1', 'all');
	}

	public function crtplst_all_assets_for_elementor_editor_admin(){
		$all_css_js_file = array(
            'crtplst_products_admin_icon_css' => array('crtplst_path_admin_define'=>CRTPLST_ASFSK_ASSETS_ADMIN_DIR_FILE . '/icon.css'),
        );
        foreach($all_css_js_file as $handle => $fileinfo){
            wp_enqueue_style( $handle, $fileinfo['crtplst_path_admin_define'], null, '2.1', 'all');
        }
	}

	public function __construct() {
		// For public assets
		add_action('wp_enqueue_scripts', [$this, 'crtplst_all_assets_for_the_public']);

		// For Elementor Editor
		add_action('elementor/editor/before_enqueue_scripts', [$this, 'crtplst_all_assets_for_elementor_editor_admin']);
		
		// Register Category
		add_action( 'elementor/elements/categories_registered', [ $this, 'crtplst_add_elementor_widget_categories' ] );

		// Register widgets
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'crtplst_register_widgets' ] );

		// Register editor scripts
		add_action( 'elementor/editor/after_enqueue_scripts', [ $this, 'crtplst_admin_editor_scripts' ] );
	}
}

// Instantiate Plugin Class
CRTPLSTCreativeListProductsClass::instance();